/**
 * Same as the sync version, but handles promises.
 *
 * @since 4.0.0
 */
export const async = (...args) => {
    const [first, ...othersFns] = args;
    return (async (arg) => {
        const firstvalue = await first(arg);
        let latestvalue = firstvalue;
        for (var i = 0; i < othersFns.length; i++) {
            latestvalue = await othersFns[i](latestvalue);
        }
        return latestvalue;
    });
};
/**
 * Creates a pipeline of synchronous functions
 *
 * @example
 *
 * ```ts
 * import { Num, Pipe } from 'tiinvo'
 *
 * const vat = Pipe.sync(Num.div(100), Num.mul(22));
 *
 * vat(100)     // 22
 * vat(150)     // 33
 * vat(200)     // 44
 * ```
 *
 * @since 4.0.0
 */
export const sync = (...args) => {
    const [first, ...othersFns] = args;
    switch (args.length) {
        case 0: return (a) => a;
        case 1: return first;
        case 2: return (arg) => args[1](first(arg));
        case 3: return (arg) => args[2](args[1](first(arg)));
        case 4: return (arg) => args[3](args[2](args[1](first(arg))));
        case 5: return (arg) => args[4](args[3](args[2](args[1](first(arg)))));
        case 6: return (arg) => args[5](args[4](args[3](args[2](args[1](first(arg))))));
        case 7: return (arg) => args[6](args[5](args[4](args[3](args[2](args[1](first(arg)))))));
        case 8: return (arg) => args[7](args[6](args[5](args[4](args[3](args[2](args[1](first(arg))))))));
        case 9: return (arg) => args[8](args[7](args[6](args[5](args[4](args[3](args[2](args[1](first(arg)))))))));
        case 10: return (arg) => args[9](args[8](args[7](args[6](args[5](args[4](args[3](args[2](args[1](first(arg))))))))));
        default: return ((arg) => othersFns.reduce((retval, fn) => fn(retval), first(arg)));
    }
};
