export function guardOf(a, x) {
    const _g = (x, y) => {
        if (Array.isArray(y)) {
            if (y.length !== x.length) {
                return false;
            }
            for (let i = 0; i < x.length; i++) {
                const z = x[i];
                if (typeof z === 'function') {
                    if (!z(y[i])) {
                        return false;
                    }
                }
                else if (typeof z === 'object' && 'guard' in z && typeof z.guard === 'function') {
                    if (!z.guard(y[i])) {
                        return false;
                    }
                }
                else {
                    return false;
                }
            }
            return true;
        }
        return false;
    };
    if (arguments.length === 2) {
        return _g(a, x);
    }
    return (z) => _g(a, z);
}
export function get(tuple, index) {
    if (arguments.length === 2 && typeof tuple !== 'number' && typeof index === 'number') {
        return tuple[index] ?? null;
    }
    return (t) => typeof tuple === 'number' ? t[tuple] ?? null : null;
}
/**
 * Returns a Tuple's length
 *
 * @example
 *
 * ```ts
 * import { Tuple } from 'tiinvo'
 *
 * Tuple.length([10, 20, 30]) // 3
 * ```
 *
 * @group accessors
 * @since 4.0.0
 */
export const length = (a) => a.length;
export function map(m, t) {
    const _map = (m, t) => {
        return m.map((x, i) => {
            if (typeof x === 'function') {
                return x(t[i]);
            }
            else if (typeof x === 'object' && !!x && typeof x === 'object' && 'map' in x && typeof x.map === 'function') {
                return x.map(t[i]);
            }
        });
    };
    if (arguments.length === 2 && Array.isArray(t)) {
        return _map(m, t);
    }
    return (y) => _map(m, y);
}
//#endregion
